﻿<#  
.SYNOPSIS  
   	Sets the configuration for the Office 2010 Filter Pack after installing Microsoft Exchange 2010
.DESCRIPTION  
	Original script by Microsoft
	Copyright (c) 2009 Microsoft Corporation. All rights reserved.
	THIS CODE IS MADE AVAILABLE AS IS, WITHOUT WARRANTY OF ANY KIND. THE ENTIRE 
	RISK OF THE USE OR THE RESULTS FROM THE USE OF THIS CODE REMAINS WITH THE USER.
	This is a filter registration script to configure Exchange Server 2010 to index Office 2007 and 2010 file attachments. 
.NOTES  
    Version      	: 1.0 - initial version
			: 2.0 - 05/08/2010 - better detection of installed filter packs; uses Filter Pack 2; includes PDF iFilter pack; menu
    Rights Required	: Local Admin on server
    Sched Task Req'd	: No
    Exchange Version	: 2010
    Author       	: Pat Richard, Exchange MVP
    Email/Blog/Twitter	: pat@innervation.com 	http://ucblogs.net/blogs/exchange @patrichard
    Dedicated Blog	: 
    Disclaimer   	: You running this script means you won't blame me if this breaks your stuff.
    Info Stolen from 	: 
.LINK  
    http://www.ucblogs.net/blogs/exchange/archive/2009/12/14/Cleaner-configuration-of-the-FilterPack-for-Exchange-Server-2010.aspx
.EXAMPLE
	.\Set-Exchange2010FilterConfig.ps1
.INPUTS
	None. You cannot pipe objects to this script.
#>

#Requires -Version 2.0

[string] $menu = @'
	
	Exchange Server 2010 - Filter Configuration script
	Please select an option from the list below
	
	1) Register the Microsoft Filter Pack 2.0
	2) Register the Adobe PDF iFilter 9.0
	3) Register both the Microsoft and Adobe filter packs

	5) Exit

Select an option.. [1-5]?
'@

[string] $DLLPath = $env:CommonProgramFiles + "\Microsoft Shared\Filters"
[string] $CLSIDKey = "HKLM:\SOFTWARE\Microsoft\ExchangeServer\V14\MSSearch\CLSID"
[string] $FiltersKey = "HKLM:\SOFTWARE\Microsoft\ExchangeServer\v14\MSSearch\Filters"

# Filter DLL Locations
[string] $officeFilterLocation = $DLLPath + "\offfiltx.dll"
[string] $onenoteFilterLocation = $DLLPath + "\ONIFilter.dll"
[string] $visioFilterLocation = $DLLPath + "\VISFilt.DLL"

function VerifyMSFilterPackInstalled(){
	$msfp = Test-Path "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\{95140000-2000-0409-1000-0000000FF1CE}"
	return $msfp
} # end VerifyMSFilterPackInstalled

function VerifyAdobeFilterPackInstalled(){
	$afp = Test-Path "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\{5EA12CF3-8162-47F6-ACAF-45AD03EFB08F}"
	return $afp
} #end VerifyAdobeFilterPackInstalled

function GetExchangeInstallPath()
{
    # Get the root setup entires.
    $setupRegistryPath = "HKLM:\SOFTWARE\Microsoft\ExchangeServer\v14\Setup"
    $setupEntries = Get-ItemProperty $setupRegistryPath
    if($setupEntries -eq $null)
    {
        return $null
    }

    # Try to get the Install Path.
    $installPath = $setupEntries.MsiInstallPath
    return $installPath
} # end GetExchangeInstallPath

function RegWrite($RegPath, $RegKey, $RegValue, $RegType){
	if (Get-ItemProperty -Name $RegKey -path $RegPath -ErrorAction SilentlyContinue) { 
		Set-ItemProperty $RegPath -Name $RegKey -Value $RegValue
	} else { 
	    	New-ItemProperty $RegPath -Name $RegKey -Value $RegValue -PropertyType $RegType		
	} 
}

function RegisterMSFilterPack(){	
	if (VerifyMSFilterPackInstalled){
		# Filter GUIDs
		$docxGuid    ="{5A98B233-3C59-4B31-944C-0E560D85E6C3}"
		$pptxGuid    ="{DDFE337F-4987-4EC8-BDE3-133FA63D5D85}"
		$xlsxGuid    ="{F90DFE0C-CBDF-41FF-8598-EDD8F222A2C8}"
		$zipGuid     ="{20E823C2-62F3-4638-96BD-90F4F6784EBC}"
		$xlsbGuid    ="{312AB530-ECC9-496E-AE0E-C9E6C5392499}"
		$onenoteGuid ="{B8D12492-CE0F-40AD-83EA-099A03D493F1}"
		$vsdGuid     ="{FAEA5B46-761B-400E-B53E-E805A97A543E}"
		
		# Create CLSIDs
		Write-Host "Creating CLSIDs..." -nonewline
		
		if (!(Test-Path $CLSIDKey\$docxGuid)){New-Item -Path $CLSIDKey -Name $docxGuid -Value $officeFilterLocation -Type String}
		if (!(Test-Path $CLSIDKey\$pptxGuid)){New-Item -Path $CLSIDKey -Name $pptxGuid  -Value $officeFilterLocation -Type String}
		if (!(Test-Path $CLSIDKey\$xlsxGuid)){New-Item -Path $CLSIDKey -Name $xlsxGuid  -Value $officeFilterLocation -Type String}
		if (!(Test-Path $CLSIDKey\$zipGuid )){New-Item -Path $CLSIDKey -Name $zipGuid  -Value $officeFilterLocation -Type String}
		if (!(Test-Path $CLSIDKey\$xlsbGuid)){New-Item -Path $CLSIDKey -Name $xlsbGuid -Value $officeFilterLocation -Type String}
		if (!(Test-Path $CLSIDKey\$oneNoteGuid)){New-Item -Path $CLSIDKey -Name $onenoteGuid -Value $onenoteFilterLocation -Type String}
		if (!(Test-Path $CLSIDKey\$vsdGuid)){New-Item -Path $CLSIDKey -Name $vsdGuid  -Value $visioFilterLocation -Type String}
		write-host "done!" -foregroundcolor green
		
		# Set Threading model
		Write-Host "Setting threading model..." -nonewline
		
		RegWrite $CLSIDKey\$docxGuid ThreadingModel Both String
		RegWrite $CLSIDKey\$pptxGuid ThreadingModel Both String
		RegWrite $CLSIDKey\$xlsxGuid ThreadingModel Both String
		RegWrite $CLSIDKey\$zipGuid ThreadingModel Both String
		RegWrite $CLSIDKey\$xlsbGuid ThreadingModel Both String
		RegWrite $CLSIDKey\$onenoteGuid ThreadingModel Both String
		RegWrite $CLSIDKey\$vsdGuid ThreadingModel Both String
		write-host "done!" -foregroundcolor green
		
		# Create Filter Entries
		Write-Host "Creating Filter Entries..." -nonewline
		
		# Uncomment these if you wish to index these uncommonly exchanged formats
		#if (!(Test-Path $FiltersKey\.docm)){New-Item -Path $FiltersKey -Name ".docm" -Value $docxGuid -Type String}
		#if (!(Test-Path $FiltersKey\.pptm)){New-Item -Path $FiltersKey -Name ".pptm" -Value $pptxGuid -Type String}
		#if (!(Test-Path $FiltersKey\.xlsm)){New-Item -Path $FiltersKey -Name ".xlsm" -Value $xlsxGuid -Type String}
		#if (!(Test-Path $FiltersKey\.vss)){New-Item -Path $FiltersKey -Name ".vss" -Value $vsdGuid   -Type String}
		#if (!(Test-Path $FiltersKey\.vst)){New-Item -Path $FiltersKey -Name ".vst" -Value $vsdGuid   -Type String}
		#if (!(Test-Path $FiltersKey\.vsx)){New-Item -Path $FiltersKey -Name ".vsx" -Value $vsdGuid   -Type String}
		#if (!(Test-Path $FiltersKey\.vtx)){New-Item -Path $FiltersKey -Name ".vtx" -Value $vsdGuid   -Type String}
		
		# These are the entries for commonly exchange formats
		if (!(Test-Path $FiltersKey\.docx)){New-Item -Path $FiltersKey -Name ".docx" -Value $docxGuid -Type String}
		if (!(Test-Path $FiltersKey\.pptx)){New-Item -Path $FiltersKey -Name ".pptx" -Value $pptxGuid -Type String}
		if (!(Test-Path $FiltersKey\.xlsx)){New-Item -Path $FiltersKey -Name ".xlsx" -Value $xlsxGuid -Type String}
		if (!(Test-Path $FiltersKey\.xlsb)){New-Item -Path $FiltersKey -Name ".xlsb" -Value $xlsbGuid -Type String}
		if (!(Test-Path $FiltersKey\.zip)){New-Item -Path $FiltersKey -Name ".zip" -Value $zipGuid  -Type String}
		if (!(Test-Path $FiltersKey\.one)){New-Item -Path $FiltersKey -Name ".one" -Value $onenoteGuid -Type String}
		if (!(Test-Path $FiltersKey\.vsd)){New-Item -Path $FiltersKey -Name ".vsd" -Value $vsdGuid   -Type String}
		write-host "done!" -foregroundcolor green
		
		Write-Host "Registry subkeys created.`n"
		Write-Host "Please restart Microsoft Search  (Exchange) service from the Services console, or by running `nstop-service msftesql-Exchange -Force ; start-service MSExchangeSearch" -foregroundcolor yellow
		$restartService = read-host 'Restart service? Please type (Y)es or (N)o...'
		switch ($restartService){
			Y {RestartSearch}
			N {Write-host 'Exiting...'}
		}
	}else{
		Write-Host "Microsoft Filter Pack 2.0 not installed. Please install and try again.`n" -foregroundcolor red -backgroundcolor black
	}
}

function RegisterAdobeFilterPack(){
	if (VerifyAdobeFilterPackInstalled){
		# Adobe iFilter Directory Path
		$iFilterDirName = "C:\Program Files\Adobe\Adobe PDF IFilter 9 for 64-bit platforms\bin"
		
		# Get the original path environment variable
		$original = (Get-ItemProperty "HKLM:SYSTEM\CurrentControlSet\Control\Session Manager\Environment" Path).Path
		
		# Add the ifilter path
		Set-ItemProperty "HKLM:SYSTEM\CurrentControlSet\Control\Session Manager\Environment" Path -value ( $original + ";" + $iFilterDirName )
		
		$CLSIDKey = "HKLM:\SOFTWARE\Microsoft\ExchangeServer\V14\MSSearch\CLSID"
		$FiltersKey = "HKLM:\SOFTWARE\Microsoft\ExchangeServer\v14\MSSearch\Filters"
		
		# Filter DLL Locations
		$pdfFilterLocation = “PDFFilter.dll"
		
		# Filter GUIDs
		$PDFGuid    ="{E8978DA6-047F-4E3D-9C78-CDBE46041603}"
		
		# Create CLSIDs
		Write-Host "Creating CLSIDs..."
		
		New-Item -Path $CLSIDKey -Name $PDFGuid -Value $pdfFilterLocation -Type String
		
		# Set Threading model
		Write-Host "Setting threading model..."
		
		New-ItemProperty -Path "$CLSIDKey\$PDFGuid" -Name "ThreadingModel" -Value "Both" -Type String
		
		# Set Flags
		Write-Host "Setting Flags..."
		New-ItemProperty -Path "$CLSIDKey\$PDFGuid" -Name "Flags" -Value "1" -Type Dword
		
		# Create Filter Entries
		Write-Host "Creating Filter Entries..."
		
		# These are the entries for commonly exchange formats
		New-Item -Path $FiltersKey -Name ".pdf" -Value $PDFGuid -Type String
		
		if ((Get-ExchangeServer (hostname)).IsHubTransportServer){
			Write-Host "`n`nRegistry subkeys created. If this server holds the Hub Transport Role, the Network Service will need to have read access to the following registry keys:`n`t$CLSIDKey\$PDFGuid`n`t$FiltersKey\.pdf `nSetting ACLs now...” -Foregroundcolor yellow -NoNewLine
			$acl= get-acl -path "$CLSIDKey\{E8978DA6-047F-4E3D-9C78-CDBE46041603}"
			$acl2= get-acl -path "$FiltersKey\.pdf"
			$inherit = [system.security.accesscontrol.InheritanceFlags]"ContainerInherit, ObjectInherit"
			$propagation = [system.security.accesscontrol.PropagationFlags]"None"
			$rule=new-object system.security.accesscontrol.registryaccessrule "NETWORK SERVICE","ReadKey",$inherit,$propagation,"Allow"
			$acl.addaccessrule($rule)
			$acl|set-acl
			$acl2.addaccessrule($rule)
			$acl2|set-acl
			Write-Host "done!" -foregroundcolor green
			# test for rights here
		}

		$RebootRequired = $true
		$ResetSearchIndexRequired = $true
	}else{
		Write-Host "Adobe iFilter Pack 9.0 not installed. Please install and try again.`n" -foregroundcolor red -backgroundcolor black
	}
}

function RestartSearch(){
	write-host "Restarting..." -NoNewLine
	stop-service msftesql-Exchange -Force
	start-service MSExchangeSearch
	write-host "done!" -foregroundcolor green
}



Do { 	
if ($RebootRequired -eq $true){write-host "`t`t`t`t`t`t`t`t`t`n`t`t`t`tREBOOT REQUIRED!`t`t`t`n`t`t`t`t`t`t`t`t`t`n`t`tDO NOT SEARCH IN EXCHANGE BEFORE REBOOTING!`t`t`n`t`tAND RUNNING .\ResetSearchIndex.ps1 -Force -All`t`t" -backgroundcolor red -foregroundcolor black}
	if ($opt -ne "None") {write-host "Last command: "$opt -foregroundcolor Yellow}	
	$opt = Read-Host $menu

	switch ($opt)    {
		1 { RegisterMSFilterPack }
		2 { RegisterAdobeFilterPack }
		3 { RegisterMSFilterPack; RegisterAdobeFilterPack }
		4 {  }
		5 { popd; write-host "Exiting..."; exit}
		default { write-host "You haven't selected any of the available options." }
	}
} while ($opt -ne 5)

